﻿using CSCore;
using CSCore.Codecs.WAV;
using System.Collections.Generic;
using System.IO;
using WaveFormat = CSCore.WaveFormat;

public class StreamMixer
{
    private List<IWaveSource> sources = new List<IWaveSource>();
    private WaveWriter writer;

    public void AddInputStream(Stream inputStream)
    {
        var waveSource = new WaveStream(inputStream, new WaveFormat(48000, 16, 2));
        lock (sources)
        {
            sources.Add(waveSource);
        }
    }

    public void WriteMixedToOutputStream(Stream outputStream)
    {
        WaveFormat format = new WaveFormat(48000, 16, 2);
        writer = new WaveWriter(outputStream, format);

        byte[] buffer = new byte[format.BytesPerSecond / 2];

        while (sources.Count > 0)
        {
            int read = 0;
            lock (sources)
            {
                foreach (var source in sources)
                {
                    int bytes = source.Read(buffer, 0, buffer.Length);
                    if (bytes == 0)
                    {
                        source.Dispose();
                        sources.Remove(source);
                    }
                    else
                    {
                        read = bytes;
                    }
                }
            }

            if (read > 0)
            {
                writer.Write(buffer, 0, read);
            }
        }

        writer.Dispose();
    }

    public void Stop()
    {
        writer?.Dispose();
        writer = null;

        lock (sources)
        {
            foreach (var source in sources)
            {
                source.Dispose();
            }

            sources.Clear();
        }
    }
}