﻿using System.Text.Json;
using Microsoft.Extensions.Caching.Memory;
using NLog;

namespace Elementary.Audio;

public class SozaiAPI : ITtsAPI
{
    private ILogger _logger;
    
    private Asset[] Assets;

    private MemoryCache _cache;

    private class Asset
    {
        /// <summary>
        /// SHA1 hash of the file
        /// </summary>
        public string hash { get; set; }

        public string id { get; set; }
        public string[] names { get; set; }
        public string[] namespaces { get; set; }
        public string path { get; set; }
        public string url { get; set; }
    }

    private HttpClient _client;

    public SozaiAPI()
    {
        _logger = LogManager.GetCurrentClassLogger();
        _cache = new MemoryCache(new MemoryCacheOptions());
    }

    public async Task Setup(string url)
    {
        _client = new HttpClient();
        var response = await _client.GetAsync(url);
        var stream = await response.Content.ReadAsStreamAsync();
        Assets = await JsonSerializer.DeserializeAsync<Asset[]>(stream);

        // Console.WriteLine($"Loaded {Assets.Length} assets");
        _logger.Info($"Loaded {Assets.Length} assets");
    }

    public async Task<Stream?> GetAudioStream(string name)
    {
        var asset = Assets.FirstOrDefault(asset => asset.names.Contains(name));
        if (asset == null) return null;

        _logger.Info($"Requested {asset.names[0]}");
        
        if (_cache.TryGetValue(asset.url, out Stream? stream))
        {
            _logger.Info($"Cache hit {asset.url}");
            return stream;
        }

        var response = await _client.GetAsync(asset.url);
        _logger.Info($"Got response {response.StatusCode}");
        stream = await response.Content.ReadAsStreamAsync();
        _cache.Set(asset.url, stream);
        return stream;
    }
}