﻿using System.Text.RegularExpressions;
using LiteDB;
using NLog;

namespace Elementary.Dictionary;

public class DictionaryDB
{
    private LiteDatabase _db;
    private ILogger _logger;
    
    private class DictionaryItem
    {
        public string Surface { get; set; }
        public string Pronunciation { get; set; }
        public int Priority { get; set; }
    }
    
    public DictionaryDB()
    {
        _logger = LogManager.GetCurrentClassLogger();
    }
    
    public async Task Setup()
    {
        _db = new LiteDatabase("dictionary.db");
    }
    
    public void Add(string surface, string pronunciation, int priority = 5)
    {
        surface = Regex.Replace(surface, @"<:[\w]+:[\d]+>", m => m.Value.Split(":")[1]); // <:emoji:123456789> -> emoji
        surface = Regex.Replace(surface, @"<a:[\w]+:[\d]+>", m => m.Value.Split(":")[1]); // <:emoji:123456789> -> emoji
        
        var collection = _db.GetCollection<DictionaryItem>("dictionary");
        
        // Remove existing items
        collection.DeleteMany(item => item.Surface == surface);
        
        collection.Insert(new DictionaryItem
        {
            Surface = surface,
            Pronunciation = pronunciation,
            Priority = priority
        });
        _logger.Info($"Added {surface} -> {pronunciation}");
    }
    
    public void Remove(string surface)
    {
        surface = Regex.Replace(surface, @"<:[\w]+:[\d]+>", m => m.Value.Split(":")[1]); // <:emoji:123456789> -> emoji
        surface = Regex.Replace(surface, @"<a:[\w]+:[\d]+>", m => m.Value.Split(":")[1]); // <:emoji:123456789> -> emoji
        
        var collection = _db.GetCollection<DictionaryItem>("dictionary");
        collection.DeleteMany(item => item.Surface == surface);
        _logger.Info($"Removed {surface}");
    }
    
    public string Replace(string text)
    {
        var collection = _db.GetCollection<DictionaryItem>("dictionary");
        var items = collection.FindAll().OrderByDescending(item => item.Priority);
        
        foreach (var item in items)
        {
            text = text.Replace(item.Surface, item.Pronunciation);
        }
        
        return text;
    }
    
    public Dictionary<string, string> GetAll()
    {
        var collection = _db.GetCollection<DictionaryItem>("dictionary");
        var items = collection.FindAll();
        
        var dict = new Dictionary<string, string>();
        foreach (var item in items)
        {
            dict.Add(item.Surface, item.Pronunciation);
        }
        
        return dict;
    }
}