﻿
using System.Text.Json;
using Microsoft.Extensions.Logging;
using NLog;
using ILogger = NLog.ILogger;

namespace Elementary.Dictionary;

public class EmojiDictionary
{
    private Dictionary<string, DictionaryItem> _dict;
    private readonly ILogger _logger;
    
    private class DictionaryItem
    {
        public string[] keywords { get; set; }
        public string short_name { get; set; }
        public string group { get; set; }
        public string subgroup { get; set; }
    }
    
    public EmojiDictionary()
    {
        _logger = LogManager.GetCurrentClassLogger();
    }
    
    public async Task Setup(string Path)
    {
        using StreamReader file = new(Path);
        _dict = await JsonSerializer.DeserializeAsync<Dictionary<string, DictionaryItem>>(file.BaseStream);
        _logger.Info($"Loaded {_dict.Count} emoji");
    }

    public string Replace(string text)
    {
        string[] ignoreKeys = {"、", "。"};
        
        foreach (var (key, value) in _dict)
        {
            if (ignoreKeys.Contains(key)) continue;
            text = text.Replace(key, value.short_name);
        }
        
        return text;
    }
}