﻿using System.Diagnostics;
using System.Text;
using System.Text.RegularExpressions;

namespace Elementary.Dictionary;

public class Mecab
{
    private string? _dictionaryPath;
    private bool _enabled;

    public async Task Setup(bool enabled, string? dictionaryPath)
    {
        _dictionaryPath = dictionaryPath;
        _enabled = enabled;
        await Task.CompletedTask;
    }

    public async Task<String> ParseToKana(string text)
    {
        if (!_enabled)
        {
            return text;
        }

        // "Apple PencilあああiPad" -> "Apple Pencil iPad"
        var englishWords = Regex.Matches(text, @"[a-zA-Z]+").Select(m => m.Value).ToList();
        var englishWordsString = string.Join(" ", englishWords);

        String args = $"-Oyomi";
        if (_dictionaryPath != null)
        {
            args += $" -d {_dictionaryPath}";
        }

        var process = new Process
        {
            StartInfo =
            {
                FileName = "mecab",
                Arguments = args,
                RedirectStandardInput = true,
                RedirectStandardOutput = true,
                StandardOutputEncoding = Encoding.UTF8,
                StandardInputEncoding = Encoding.UTF8,
                RedirectStandardError = true,
                UseShellExecute = false,
                CreateNoWindow = true
            }
        };
        process.Start();
        await process.StandardInput.WriteLineAsync(englishWordsString);
        await process.StandardInput.FlushAsync();
        process.StandardInput.Close();
        var result = await process.StandardOutput.ReadToEndAsync();
        process.WaitForExit();

        // replace english words with kana
        var kanaWords = result.Split(" ").Select(m => m.Trim()).ToList();
        for (var i = 0; i < englishWords.Count; i++)
        {
            text = text.Replace(englishWords[i], kanaWords[i]);
        }

        return text;
    }
}